@@ -1,4 +1,4 @@
-"rad.cov" <-
+"Rad.cov" <-
 function (x1, x2, p = 1, with.log = TRUE, with.constant = TRUE, 
             C = NA,marginal=FALSE) 
 {
